/*
 * lcd_hd44780.c
 *
 * Created : 01.11.2014 15:26:25
 * Modified: 30.01.2015 21:38:43
 *
 * Version : 1.0
 *
 * Author: HB9FGK - Laurent Ferrari
 *
 * Plateforme : uC ATMEL AVR ATtiny & ATmega
 *
 * Description :
 *              Fonctions d'initialisation et d'affichage pour LCD 
 *               base de controlleur HD44780 en mode 4 bits.
 *
 *              ! Ne pas oublier d'initialiser les ports E/S concerns. !
 */ 

#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>
#include "lcd_hd44780.h"

#define DATA_MASK (0x0f<<DATA_BIT)
#define COMP_MASK (~DATA_MASK)

void lcd_init_write(unsigned char cmd){
	

	// Select the command register

	RS_PORT &= ~(1<<RS_BIT);
	_delay_ms(1);
	DATA_PORT = ((cmd>>4)<<DATA_BIT)|(DATA_PORT&COMP_MASK);


	EN_PORT |= (1<<EN_BIT);  // EN = 1
	_delay_ms(1);
	EN_PORT &= ~(1<<EN_BIT);
	
}


void lcd_putcmd(unsigned char cmd){
	
	
	// Select the command register
	RS_PORT &= ~(1<<RS_BIT);
    _delay_us(1000);
	
	//High nible first
	DATA_PORT = (((cmd&0xf0)>>4)<<DATA_BIT)|(DATA_PORT&COMP_MASK);
	EN_PORT |= (1<<EN_BIT);  // E = 1
	_delay_us(1000);
	EN_PORT &= ~(1<<EN_BIT); // E = 0
	_delay_us(500);
	
	
	//Low nible
	DATA_PORT = (((cmd&0x0f)<<DATA_BIT)|(DATA_PORT&COMP_MASK));
	EN_PORT |= (1<<EN_BIT);  // E = 1
	_delay_us(1000);
	EN_PORT &= ~(1<<EN_BIT);	 // E = 0

}

void lcd_setdisplay(unsigned char settings){

   lcd_putcmd(0x08|settings);
}


void init_lcd(void){
	
	
	
	_delay_ms(100);
	lcd_init_write(0x30);
	_delay_ms(10);
	lcd_init_write(0x30);
	_delay_ms(1);
	lcd_init_write(0x30);
	_delay_ms(1);
	lcd_init_write(0x20); // Passage en mode 4 bits
	_delay_ms(1);
	
	lcd_putcmd(0x28); // 4 bits, 2 lignes, 5x8 pixels
	_delay_ms(1);
    lcd_setdisplay(LCD_ON|CURSOR_OFF|CURSOR_NOBLINK); // Affichage visible, curseur invisible, curseur fixe
	_delay_ms(1);
	lcd_putcmd(0x01); // Effacement de l'cran
	_delay_ms(1);
	lcd_putcmd(0x06); // Depl. curseur vers la droite, pas decalage du compteur d'adresse
	_delay_ms(100);	
}





void lcd_putchar(char car){
	

	// Set RS for write in DDRAM
	RS_PORT |= (1<<RS_BIT);

	_delay_us(100);
	
	//High nible first
	DATA_PORT = (((car&0xf0)>>4)<<DATA_BIT)|(DATA_PORT&COMP_MASK);
	EN_PORT |= (1<<EN_BIT);  // E = 1

	_delay_us(100);
	EN_PORT &= ~(1<<EN_BIT); // E = 0

	_delay_us(100);
	
	//Low nible
	DATA_PORT = (((car&0x0f)<<DATA_BIT)|(DATA_PORT&COMP_MASK));
	EN_PORT |= (1<<EN_BIT);  // E = 1

	_delay_us(100);
	EN_PORT &= ~(1<<EN_BIT);	 // E = 0
	
	
}

void lcd_putstr(char *text){
	
	int i;
	for (i = 0 ; text[i]!=0x00 ; i++)
	  lcd_putchar(text[i]);
	
}

void lcd_gotoxy(unsigned char x, unsigned char y){
	
	unsigned char position;
	
	position = x+0x80;
	if (y) position += 0x40;
	
	lcd_putcmd(position);
	
	
	
}

void lcd_clrscr(void){
	
	lcd_putcmd(0x01);
	
}
